<?php

namespace SGalinski\SgJobs\Service;

/***************************************************************
 *  Copyright notice
 *
 *  (c) sgalinski Internet Services (https://www.sgalinski.de)
 *
 *  All rights reserved
 *
 *  This script is part of the TYPO3 project. The TYPO3 project is
 *  free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  The GNU General Public License can be found at
 *  http://www.gnu.org/copyleft/gpl.html.
 *
 *  This script is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  This copyright notice MUST APPEAR in all copies of the script!
 ***************************************************************/

use SGalinski\SgJobs\Domain\Repository\JobRepository;
use TYPO3\CMS\Core\Database\ConnectionPool;
use TYPO3\CMS\Core\Database\Query\Restriction\DeletedRestriction;
use TYPO3\CMS\Core\Database\Query\Restriction\FrontendRestrictionContainer;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Object\ObjectManager;

/**
 * Frontend filter service class
 */
class FrontendFilterService {

	/**
	 * Get all the jobs based on the given filters
	 *
	 * @param array $filters
	 * @param int $recordPageId
	 * @param int $limit
	 * @param int $offset
	 * @return mixed
	 * @throws \InvalidArgumentException
	 */
	public static function getJobs(array $filters = [], $recordPageId, $limit = 0, $offset = 0): array {
		// get all company ids
		$queryBuilder = GeneralUtility::makeInstance(ConnectionPool::class)->getQueryBuilderForTable(
			'tx_sgjobs_domain_model_job'
		);

		$statement = $queryBuilder->select('a.uid')
			->from('tx_sgjobs_domain_model_job', 'a')
			->join(
				'a', 'tx_sgjobs_domain_model_company', 'b'
				, $queryBuilder->expr()->eq('a.company', 'b.uid')
			);

		$queryBuilder
			->getRestrictions()
			->removeAll()
			->add(GeneralUtility::makeInstance(DeletedRestriction::class))
			->add(GeneralUtility::makeInstance(FrontendRestrictionContainer::class));

		if ($filters['filterCountry'] !== '0' && $filters['filterCountry'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'b.country', $queryBuilder->createNamedParameter($filters['filterCountry'])
				)
			);
		}

		if ($filters['filterLocation'] !== '0' && $filters['filterLocation'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'b.city', $queryBuilder->createNamedParameter($filters['filterLocation'])
				)
			);
		}

		if ($filters['filterCompany'] !== '0' && $filters['filterCompany'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'b.name', $queryBuilder->createNamedParameter($filters['filterCompany'])
				)
			);
		}

		if ($filters['filterArea'] !== '0' && $filters['filterArea'] !== NULL) {
			$statement->andWhere(
				$queryBuilder->expr()->eq(
					'a.area', $queryBuilder->createNamedParameter($filters['filterArea'])
				)
			);
		}

		$statement->andWhere(
			$queryBuilder->expr()->eq(
				'b.pid', $queryBuilder->createNamedParameter($recordPageId)
			)
		);

		$companies = $statement->execute()->fetchAll();

		$result = [];
		$result[0] = '';
		foreach ($companies as $company) {
			$result[$company['uid']] = $company['uid'];
		}

		// use company ids to filter the jobs
		$objectManager = GeneralUtility::makeInstance(ObjectManager::class);
		/** @var JobRepository $jobRepository */
		$jobRepository = $objectManager->get(JobRepository::class);

		// when filters are set, but there are no companies found, return an empty result
		if (empty(!$filters) && empty($companies)) {
			return $companies;
		}

		// return the filtered jobs
		return $jobRepository->findByJobIds($recordPageId, $result, $limit, $offset)->toArray();
	}
}
